/*
 *  Richard Bensonizer plugin for gaim
 *  Copyright (C) 2006  Michele Baldessari <michele@pupazzo.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public License as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* config.h */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* system headers */
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/* libgcrypt headers */
#include <gcrypt.h>

/* gaim headers */
#include "gaim.h"
#include "notify.h"
#include "version.h"
#include "util.h"
#include "debug.h"
#include "core.h"

#ifdef USING_GTK
/* gaim GTK headers */
#include "gtkplugin.h"
#include "gtkconv.h"
#endif

/* gaim-benson headers */
#include "benson-plugin.h"
#include "str.h"


GaimPlugin *benson_plugin_handle;

#define WEBSITE "http://michele.pupazzo.org"

static const gchar* 
bensonize(const gchar *text)
{
	struct mystr tmp = INIT_MYSTR;
	const char *ret;

	str_alloc_text(&tmp, text);
	str_replace_text(&tmp, "ici", "iSci");
	str_replace_text(&tmp, "ice", "iSce");
	str_replace_text(&tmp, "aci", "aSci");
	str_replace_text(&tmp, "oci", "oSci");
	str_replace_text(&tmp, "eci", "eSci");
	str_replace_text(&tmp, "uci", "uSci");
	str_replace_text(&tmp, "ace", "aSce");
	str_replace_text(&tmp, "ece", "eSce");
	str_replace_text(&tmp, " il ", " er ");
	str_replace_text(&tmp, "il ", "er ");
	str_replace_text(&tmp, "gl", "j");
	ret = str_strdup(&tmp);
	str_free(&tmp);

	return (ret);
}

static void
message_send_cb(GtkWidget *widget, GtkTextBuffer *buffer)
{
        GtkTextIter start, end;
	gchar *text, *btext;

        gtk_text_buffer_get_start_iter(buffer, &start);
        gtk_text_buffer_get_end_iter(buffer, &end);
	text = gtk_text_buffer_get_text(buffer,
					&start,
					&end,
					FALSE);

	gaim_debug(GAIM_DEBUG_INFO, "benson", "Text: %s\n", text);
	btext = strdup(bensonize(text));
	gtk_text_buffer_set_text(buffer,
				 btext,
				 -1);

	g_free(text);
	return;
}


static void
conversation_new_attach(GaimConversation *conv)
{
	GaimGtkConversation *gtkconv;
	GtkTextView *view;
	GtkTextBuffer *buffer;

	gtkconv = GAIM_GTK_CONVERSATION(conv);
	view = GTK_TEXT_VIEW(gtkconv->entry);
	buffer = gtk_text_view_get_buffer(view);

        g_signal_connect(G_OBJECT(gtkconv->entry), "message_send",
                         G_CALLBACK(message_send_cb), buffer);

	return;
}

static gboolean
plugin_load(GaimPlugin *plugin)
{
        void *conv_handle = gaim_conversations_get_handle();
        GList *convs;

        gaim_debug(GAIM_DEBUG_INFO, "benson", "benson plugin loaded.\n");

        /* Attach to existing conversations */
        for (convs = gaim_get_conversations(); convs != NULL; convs = convs->next)
        {    
                conversation_new_attach((GaimConversation *)convs->data);
        }    

        gaim_signal_connect(conv_handle, "conversation-created",
                            plugin, GAIM_CALLBACK(conversation_new_attach), NULL);

        return TRUE;
}

static gboolean
plugin_unload(GaimPlugin *plugin)
{                                                                                                                                                                                                                                            
        GList *convs;

        /* Detach from existing conversations */
        for (convs = gaim_get_conversations(); convs != NULL; convs = convs->next)
        {    
                GaimGtkConversation *gtkconv = GAIM_GTK_CONVERSATION((GaimConversation *)convs->data);

                g_signal_handlers_disconnect_by_func(gtkconv->entry, message_send_cb, NULL);
        }    

        gaim_debug(GAIM_DEBUG_INFO, "benson", "benson plugin unloaded.\n");

        return TRUE;
}

static GaimPluginInfo info =
{
        GAIM_PLUGIN_MAGIC,
        GAIM_MAJOR_VERSION,
        GAIM_MINOR_VERSION,
        GAIM_PLUGIN_STANDARD,                             /**< type           */
        NULL,                                             /**< ui_requirement */
        0,                                                /**< flags          */
        NULL,                                             /**< dependencies   */
        GAIM_PRIORITY_DEFAULT,                            /**< priority       */

        NULL,                                             /**< id             */
        "Bensonizer Plugin",                              /**< name           */
        GAIM_BENSON_VERSION,                              /**< version        */
                                                          /**  summary        */
        "The Bensonizer plugin.",
                                                          /**  description    */
        "This is the Bensonizer plugin.",
        "Michele Baldessari <michele@pupazzo.org",        /**< author         */
        WEBSITE,                                          /**< homepage       */

        plugin_load,                                      /**< load           */
        plugin_unload,                                    /**< unload         */
        NULL,                                             /**< destroy        */

        NULL,                                             /**< ui_info        */
        NULL,                                             /**< extra_info     */
        NULL,
        NULL
};

static void
init_plugin(GaimPlugin *plugin)
{
}

GAIM_INIT_PLUGIN(simple, init_plugin, info)

